import React from 'react';
import ReactDOM from 'react-dom';
import { scaleBand, scaleLinear, max, format } from 'd3';

import { useData } from './useData';
import { Marks } from './Marks';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { ZeroLine } from './ZeroLine';

const width = 960;
const height = 500;
const margin = { top: 20, right: 30, bottom: 80, left: 250 };
const innerWidth = width - margin.right - margin.left;
const innerHeight = height - margin.top - margin.bottom;
const xLabelOffset = 25;
const xTitleOffset = 60;
const yLabelOffset = 15;

const App = () => {
    const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const yValue = (d) => d.country;
  const xValue = (d) => d.population;

  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight])
    .paddingInner(0.2)
    .paddingOuter(0.3);

  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]);

  const xFormatSpec = format('.2s');
  const xLabelFormat = (s) => xFormatSpec(s).replace('G', 'B');

  return (
    <div className="App">
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left}, ${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            xLabelOffset={xLabelOffset}
            xLabelFormat={xLabelFormat}
            xTitleOffset={xTitleOffset}
          />
          <AxisLeft yScale={yScale} yLabelOffset={yLabelOffset} />
          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            xLabelFormat={xLabelFormat}
          />
          <ZeroLine innerHeight={innerHeight} />
          <text
            className="x-title"
            x={innerWidth / 2}
            y={innerHeight + xTitleOffset}
          >
            Population
          </text>
        </g>
      </svg>
    </div>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
